package net.knarcraft.minecraftserverlauncher.server.servertypes;

import net.knarcraft.minecraftserverlauncher.server.ServerVersionContainer;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;

import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.downloadFile;
import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.readFile;
import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.stringBetween;

/**
 * This class represents the BungeeCord proxy server type
 */
public class BungeeCord extends AbstractServerType {

    private final String versionURL;
    private final String srcStart;
    private final String srcEnd;
    private final String oldVersionsURL;
    private final String oldVersionsPrefix;

    /**
     * Instantiates a new BungeeCord server type
     *
     * @param typeName    <p>The name of the server type</p>
     * @param isProxy     <p>Whether this server type is a proxy server</p>
     * @param versions    <p>The available versions for the server type</p>
     * @param versionURL  <p>The URL used to finding the newest version</p>
     * @param srcStart    <p>The string after which the version id starts</p>
     * @param srcEnd      <p>The string marking the end of the version id</p>
     * @param downloadURL <p>The URL used for downloading the latest version</p>
     */
    public BungeeCord(String typeName, boolean isProxy, String[] versions, String versionURL, String srcStart,
                      String srcEnd, String downloadURL, String oldVersionsURL, String oldVersionsPrefix) {
        super(typeName, isProxy, versions, downloadURL);
        this.versionURL = versionURL;
        this.srcStart = srcStart;
        this.srcEnd = srcEnd;
        this.oldVersionsURL = oldVersionsURL;
        this.oldVersionsPrefix = oldVersionsPrefix;
    }

    @Override
    public boolean downloadJar(String folder, String version) throws IOException {
        String targetFile = this.getName() + version + ".jar";
        File filePath = new File(folder + targetFile);

        if (version.equals("Latest")) {
            return downloadLatestJar(filePath);
        } else {
            String file = oldVersionsPrefix + version + ".jar";
            return filePath.isFile() || downloadFile(oldVersionsURL + file, Paths.get(filePath.toURI()));
        }
    }

    /**
     * Downloads the latest BungeeCord .jar file
     *
     * @param filePath <p>The file path to download the new jar to</p>
     * @return <p>True if the jar exists or was successfully downloaded</p>
     * @throws IOException <p>If unable to download the new jar</p>
     */
    private boolean downloadLatestJar(File filePath) throws IOException {
        ServerVersionContainer versionContainer = ServerVersionContainer.getInstance();

        String newestVersion = stringBetween(readFile(versionURL), srcStart, srcEnd);
        String oldVersion = versionContainer.getBungeeVersion();
        //The file is already the newest version
        if (filePath.isFile() && newestVersion.equals(oldVersion)) {
            return true;
        }
        //The new jar file could not be downloaded
        if (!downloadFile(downloadURL, Paths.get(filePath.toURI()))) {
            return false;
        }
        versionContainer.setBungeeVersion(newestVersion);
        return true;
    }

}
