package net.knarcraft.minecraftserverlauncher.server.servertypes;

import net.knarcraft.minecraftserverlauncher.server.ServerVersionContainer;

import java.io.File;
import java.io.IOException;
import java.nio.file.Paths;
import java.util.function.BiConsumer;
import java.util.function.Function;

import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.downloadFile;
import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.readFile;
import static net.knarcraft.minecraftserverlauncher.utility.CommonFunctions.stringBetween;

/**
 * This class represents the Travertine proxy server type
 */
public class Waterfall extends AbstractServerType {

    private final String srcStart;
    private final String srcEnd;
    private final String versionURL;
    Function<String, String> oldVersionFunction;
    BiConsumer<String, String> versionUpdateFunction;
    final ServerVersionContainer serverVersionContainer;

    /**
     * Instantiates a new Waterfall server type
     *
     * @param typeName    <p>The name of the server type</p>
     * @param isProxy     <p>Whether this server type is a proxy server</p>
     * @param versions    <p>The available versions for the server type</p>
     * @param versionURL  <p>The URL used to finding the newest version</p>
     * @param srcStart    <p>The string after which the version id starts</p>
     * @param srcEnd      <p>The string marking the end of the version id</p>
     * @param downloadURL <p>The URL used for downloading the latest version</p>
     */
    public Waterfall(String typeName, boolean isProxy, String[] versions, String versionURL, String srcStart,
                     String srcEnd, String downloadURL) {
        super(typeName, isProxy, versions, downloadURL);
        this.serverVersionContainer = ServerVersionContainer.getInstance();
        this.srcStart = srcStart;
        this.srcEnd = srcEnd;
        this.versionURL = versionURL;
        this.oldVersionFunction = serverVersionContainer::getWaterfallVersion;
        this.versionUpdateFunction = serverVersionContainer::setWaterfallVersion;
    }

    @Override
    public boolean downloadJar(String folder, String version) throws IOException {
        String file = this.getName() + version + ".jar";
        File filePath = new File(folder + file);
        String newestVersion = stringBetween(readFile(versionURL + version), srcStart, srcEnd);
        String fullURL = downloadURL + version + "/" + newestVersion + "/download";
        String oldVersion = oldVersionFunction.apply(version);
        //The file is already the newest version
        if (filePath.isFile() && newestVersion.equals(oldVersion)) {
            return true;
        }
        //The new jar file could not be downloaded
        if (!downloadFile(fullURL, Paths.get(filePath.toURI()))) {
            return false;
        }
        versionUpdateFunction.accept(version, newestVersion);
        return true;
    }
}
