package net.knarcraft.minecraftserverlauncher.utility;

import net.knarcraft.minecraftserverlauncher.Main;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.file.Paths;

public class JarBuilder {

    private String downloadedBuildToolsVersion = "#124";
    private final String buildDirectory;
    private final String jarDirectory;

    public JarBuilder(String buildDirectory, String jarDirectory) {
        this.buildDirectory = buildDirectory;
        this.jarDirectory = jarDirectory;
    }

    /**
     * Builds the latest version of the spigot .jar file
     */
    public void buildSpigotJar() {
        downloadBuildTools();
        ProcessBuilder processBuilder = new ProcessBuilder("C:\\Program Files\\Java\\jdk-16.0.1\\bin\\java.exe", "-jar", "BuildTools.jar", "--rev",
                "latest", "--output-dir", jarDirectory);
        executeBuildProcess(processBuilder);
        System.out.println("Finished building spigot .jar. Moving it to the correct location");
        moveBuiltJar("spigot-", "SpigotLatest.jar");
        System.out.println("Finished moving spigot .jar");
    }

    /**
     * Builds the latest version of the craftbukkit .jar file
     */
    public void buildBukkitJar() {
        downloadBuildTools();
        ProcessBuilder processBuilder = new ProcessBuilder("C:\\Program Files\\Java\\jdk-16.0.1\\bin\\java.exe", "-jar", "BuildTools.jar", "--compile",
                "craftbukkit", "--rev", "latest", "--output-dir", jarDirectory);
        executeBuildProcess(processBuilder);
        System.out.println("Finished building craftbukkit .jar. Moving it to the correct location");
        moveBuiltJar("craftbukkit-", "BukkitLatest.jar");
        System.out.println("Finished moving craftbukkit .jar");
    }

    /**
     * Downloads the latest BuildTools version
     */
    public void downloadBuildTools() {
        try {
            String latestVersion = getLatestBuildToolsVersion();
            if (!latestVersion.equals(downloadedBuildToolsVersion)) {
                boolean success = CommonFunctions.downloadFile("https://hub.spigotmc.org/jenkins/job/BuildTools/" +
                        "lastSuccessfulBuild/artifact/target/BuildTools.jar", Paths.get(buildDirectory + "BuildTools.jar"));
                if (!success) {
                    Main.getController().getGUI().setStatus("Unable to download the latest BuildTools");
                }
            }
        } catch (IOException e) {
            Main.getController().getGUI().setStatus("Unable to download the latest BuildTools");
        }
    }

    /**
     * Moves a built .jar file to its target file
     * @param searchString <p>The start of the name of the built file used to find it</p>
     * @param newName <p>The new name of the built file</p>
     */
    private void moveBuiltJar(String searchString, String newName) {
        File[] foundFiles = new File(jarDirectory).listFiles((file) -> file.isFile() && file.getName().startsWith(searchString));
        if (foundFiles != null && foundFiles.length == 1) {
            File newFileLocation = new File(jarDirectory + File.separator + newName);
            if (newFileLocation.exists()) {
                if (!newFileLocation.delete()) {
                    throw new IllegalArgumentException("Unable to delete previously built .jar");
                }
            }
            if (!foundFiles[0].renameTo(newFileLocation)) {
                throw new IllegalArgumentException("Unable to move built .jar");
            }
        }
    }

    /**
     * Starts the build process and initializes
     * @param processBuilder <p>The process builder to execute</p>
     */
    private void executeBuildProcess(ProcessBuilder processBuilder) {
        processBuilder.directory(new File(buildDirectory));
        processBuilder.redirectErrorStream(true);
        try {
            Process process = processBuilder.start();
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(System.out));
            BufferedReader reader = new BufferedReader(new InputStreamReader(process.getInputStream()));

            do {
                writer.write(CommonFunctions.readBufferedReader(reader));
            } while (process.isAlive());

            writer.flush();

            if (process.exitValue() == 0) {
                System.out.println("Jar building process finished successfully.");
            } else {
                throw new RuntimeException("Jar building process failed with exit code " + process.exitValue() + ".");
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Gets the latest build tools version available
     * @return <p>The latest build tools version available</p>
     * @throws IOException <p>If unable to read the version file</p>
     */
    String getLatestBuildToolsVersion() throws IOException {
        String versionDocument;
        versionDocument = CommonFunctions.readFile("https://hub.spigotmc.org/jenkins/job/BuildTools/lastSuccessfulBuild/");
        return CommonFunctions.stringBetween(versionDocument, "BuildTools #", " [");
    }

}
